"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SmartInput = void 0;
var react_1 = __importStar(require("react"));
var slate_1 = require("slate");
var slate_history_1 = require("slate-history");
var slate_react_1 = require("slate-react");
var SmartInputFetcher_1 = require("./SmartInputFetcher");
function SmartInput(props) {
    var editor = react_1.default.useMemo(function () { return (0, slate_history_1.withHistory)((0, slate_react_1.withReact)((0, slate_1.createEditor)())); }, []);
    var _a = react_1.default.useState(''), completion = _a[0], setCompletion = _a[1];
    var acceptSuggestion = function () {
        acceptCompletion(editor, completion);
        setCompletion('');
    };
    var addSuggestion = function (completion) {
        if (props.value) {
            addCompletion(editor, completion);
            setCompletion(completion);
        }
    };
    var clearSuggestion = function () {
        removeCompletion(editor);
        setCompletion('');
    };
    var fetcher = react_1.default.useState(new SmartInputFetcher_1.SmartInputFetcher(function (text, suggestion) {
        addSuggestion(suggestion);
    }, props.modelId, props.modelId ? props.mockSuggestion : function () { return ' -- sample suggestion --'; }))[0];
    fetcher.addSuggestion = function (text, suggestion) {
        var currText = props.value;
        if (suggestion && (text + suggestion).startsWith(currText)) {
            addSuggestion(suggestion);
        }
    };
    var initialValue = [
        {
            type: 'text',
            children: [{ text: props.value, type: undefined }],
        },
    ];
    (0, react_1.useEffect)(function () {
        if (props.editorRef) {
            props.editorRef.current = __assign(__assign({}, slate_react_1.ReactEditor.toDOMNode(editor, editor)), { focus: function () {
                    setTimeout(function () {
                        slate_react_1.ReactEditor.focus(editor);
                        if (props.value) {
                            slate_1.Transforms.select(editor, slate_1.Editor.end(editor, []));
                        }
                        else {
                            slate_1.Transforms.select(editor, slate_1.Editor.start(editor, []));
                        }
                    }, 50);
                } });
        }
    });
    var renderElement = react_1.default.useCallback(function (p) {
        switch (p.element.type) {
            case 'text':
                return props.renderText(__assign({}, p));
            case 'completion':
                return props.renderSuggestion(__assign(__assign({}, p), { contentEditable: 'false', suppressContentEditableWarning: true }));
            default:
                return react_1.default.createElement(slate_react_1.DefaultElement, __assign({}, p));
        }
    }, [props.renderText, props.renderSuggestion]);
    var renderPlaceholder = react_1.default.useCallback(function (p) {
        return props.renderPlaceholder(__assign(__assign(__assign({}, p), (p.attributes || {})), { style: __assign({ position: 'absolute', pointerEvents: 'none', width: '100%', maxWidth: '100%', userSelect: 'none', textDecoration: 'none' }, (p.style || {})) }));
    }, [props.renderPlaceholder]);
    return (react_1.default.createElement(slate_react_1.Slate, { editor: editor, value: initialValue, onChange: function (value) {
            var text = serialize(value.filter(function (n) { return n.type !== 'completion'; }));
            fetcher.fetch(text, props.context || {});
            props.onChange(text);
        } },
        react_1.default.createElement(slate_react_1.Editable, { placeholder: props.placeholder, className: props.className, style: __assign(__assign({}, (props.style || {})), { whiteSpace: props.multiline ? undefined : 'pre' }), onPaste: function (event) {
                clearSuggestion();
                event.preventDefault();
                var value = event.clipboardData.getData('Text');
                if (!props.multiline) {
                    editor.insertText(value.replaceAll('\n', ''));
                }
                else {
                    editor.insertText(value);
                }
            }, renderPlaceholder: props.renderPlaceholder ? renderPlaceholder : undefined, renderElement: renderElement, onBlur: function (e) {
                clearSuggestion();
                fetcher.pendingCompletion = null;
                if (props.onBlur) {
                    props.onBlur(e);
                }
            }, onFocus: function (e) {
                if (props.onFocus) {
                    props.onFocus(e);
                    if (editor.selection == null) {
                        if (props.value) {
                            slate_1.Transforms.select(editor, slate_1.Editor.end(editor, []));
                        }
                        else {
                            slate_1.Transforms.select(editor, slate_1.Editor.start(editor, []));
                        }
                    }
                }
            }, onKeyDown: function (event) {
                var hasCompletion = completion.trim().length > 0;
                if (event.key === 'Tab' && hasCompletion) {
                    event.preventDefault();
                    acceptSuggestion();
                    fetcher.pendingCompletion = null;
                }
                else if (event.key === 'ArrowRight' && hasCompletion) {
                    event.preventDefault();
                    acceptSuggestion();
                    fetcher.pendingCompletion = null;
                }
                else if (event.key === 'ArrowLeft' && hasCompletion) {
                    clearSuggestion();
                    fetcher.pendingCompletion = null;
                }
                else if ((event.key === 'ArrowDown' || event.key === 'ArrowUp') && hasCompletion) {
                    clearSuggestion();
                }
                else if (event.key === 'Backspace') {
                    fetcher.pendingCompletion = null;
                    clearSuggestion();
                }
                else if (event.key === 'Enter' && !event.metaKey) {
                    event.preventDefault();
                    if (props.multiline) {
                        clearSuggestion();
                        editor.insertText('\n');
                    }
                }
                else if (event.key !== 'Shift') {
                    if (completion && !event.metaKey && completion.startsWith(event.key) && event.key.length === 1) {
                        updateCompletion(editor, completion.substring(1));
                        setCompletion(completion.substring(1));
                    }
                    else {
                        fetcher.pendingCompletion = null;
                        clearSuggestion();
                    }
                }
                if (props.onKeyDown) {
                    props.onKeyDown(event);
                }
            } })));
}
exports.SmartInput = SmartInput;
function isCompletionActive(editor) {
    var completion = editor.children.find(function (n) { return n.type === 'completion'; });
    return !!completion;
}
function isCursorAtEndOfDocument(editor) {
    var sel = editor.selection;
    if (!sel) {
        return false;
    }
    var node = sel.anchor.path[0];
    var offset = sel.anchor.offset;
    if (node !== editor.children.length - 1) {
        return false;
    }
    return offset === slate_1.Node.string(editor.children[node]).length;
}
function acceptCompletion(editor, completion) {
    var isActive = isCompletionActive(editor);
    if (isActive) {
        removeCompletion(editor);
        editor.insertText(completion);
    }
}
function addCompletion(editor, completion) {
    var isActive = isCompletionActive(editor);
    var isCursorEnd = isCursorAtEndOfDocument(editor);
    if (!isActive && isCursorEnd) {
        slate_1.Transforms.insertNodes(editor, {
            type: 'completion',
            children: [{ text: completion }],
        }, {
            at: [editor.children.length],
        });
    }
}
function removeCompletion(editor) {
    var isActive = isCompletionActive(editor);
    if (isActive) {
        slate_1.Transforms.removeNodes(editor, {
            at: [editor.children.length - 1],
        });
    }
}
function updateCompletion(editor, newCompletion) {
    var isActive = isCompletionActive(editor);
    if (isActive && newCompletion.length > 0) {
        slate_1.Transforms.delete(editor, {
            at: {
                anchor: { path: [editor.children.length - 1, 0], offset: 0 },
                focus: { path: [editor.children.length - 1, 0], offset: 1 },
            },
        });
    }
}
var serialize = function (nodes) {
    return nodes.map(function (n) { return slate_1.Node.string(n); }).join('\n');
};
//# sourceMappingURL=SmartInput.js.map