var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import React, { useEffect } from 'react';
import { createEditor, Editor, Node, Transforms } from 'slate';
import { withHistory } from 'slate-history';
import { DefaultElement, Editable, ReactEditor, Slate, withReact } from 'slate-react';
import { SmartInputFetcher } from './SmartInputFetcher';
export function SmartInput(props) {
    var editor = React.useMemo(function () { return withHistory(withReact(createEditor())); }, []);
    var _a = React.useState(''), completion = _a[0], setCompletion = _a[1];
    var acceptSuggestion = function () {
        acceptCompletion(editor, completion);
        setCompletion('');
    };
    var addSuggestion = function (completion) {
        if (props.value) {
            addCompletion(editor, completion);
            setCompletion(completion);
        }
    };
    var clearSuggestion = function () {
        removeCompletion(editor);
        setCompletion('');
    };
    var fetcher = React.useState(new SmartInputFetcher(function (text, suggestion) {
        addSuggestion(suggestion);
    }, props.modelId, props.modelId ? props.mockSuggestion : function () { return ' -- sample suggestion --'; }))[0];
    fetcher.addSuggestion = function (text, suggestion) {
        var currText = props.value;
        if (suggestion && (text + suggestion).startsWith(currText)) {
            addSuggestion(suggestion);
        }
    };
    var initialValue = [
        {
            type: 'text',
            children: [{ text: props.value, type: undefined }],
        },
    ];
    useEffect(function () {
        if (props.editorRef) {
            props.editorRef.current = __assign(__assign({}, ReactEditor.toDOMNode(editor, editor)), { focus: function () {
                    setTimeout(function () {
                        ReactEditor.focus(editor);
                        if (props.value) {
                            Transforms.select(editor, Editor.end(editor, []));
                        }
                        else {
                            Transforms.select(editor, Editor.start(editor, []));
                        }
                    }, 50);
                } });
        }
    });
    var renderElement = React.useCallback(function (p) {
        switch (p.element.type) {
            case 'text':
                return props.renderText(__assign({}, p));
            case 'completion':
                return props.renderSuggestion(__assign(__assign({}, p), { contentEditable: 'false', suppressContentEditableWarning: true }));
            default:
                return React.createElement(DefaultElement, __assign({}, p));
        }
    }, [props.renderText, props.renderSuggestion]);
    var renderPlaceholder = React.useCallback(function (p) {
        return props.renderPlaceholder(__assign(__assign(__assign({}, p), (p.attributes || {})), { style: __assign({ position: 'absolute', pointerEvents: 'none', width: '100%', maxWidth: '100%', userSelect: 'none', textDecoration: 'none' }, (p.style || {})) }));
    }, [props.renderPlaceholder]);
    return (React.createElement(Slate, { editor: editor, value: initialValue, onChange: function (value) {
            var text = serialize(value.filter(function (n) { return n.type !== 'completion'; }));
            fetcher.fetch(text, props.context || {});
            props.onChange(text);
        } },
        React.createElement(Editable, { placeholder: props.placeholder, className: props.className, style: __assign(__assign({}, (props.style || {})), { whiteSpace: props.multiline ? undefined : 'pre' }), onPaste: function (event) {
                clearSuggestion();
                event.preventDefault();
                var value = event.clipboardData.getData('Text');
                if (!props.multiline) {
                    editor.insertText(value.replaceAll('\n', ''));
                }
                else {
                    editor.insertText(value);
                }
            }, renderPlaceholder: props.renderPlaceholder ? renderPlaceholder : undefined, renderElement: renderElement, onBlur: function (e) {
                clearSuggestion();
                fetcher.pendingCompletion = null;
                if (props.onBlur) {
                    props.onBlur(e);
                }
            }, onFocus: function (e) {
                if (props.onFocus) {
                    props.onFocus(e);
                    if (editor.selection == null) {
                        if (props.value) {
                            Transforms.select(editor, Editor.end(editor, []));
                        }
                        else {
                            Transforms.select(editor, Editor.start(editor, []));
                        }
                    }
                }
            }, onKeyDown: function (event) {
                var hasCompletion = completion.trim().length > 0;
                if (event.key === 'Tab' && hasCompletion) {
                    event.preventDefault();
                    acceptSuggestion();
                    fetcher.pendingCompletion = null;
                }
                else if (event.key === 'ArrowRight' && hasCompletion) {
                    event.preventDefault();
                    acceptSuggestion();
                    fetcher.pendingCompletion = null;
                }
                else if (event.key === 'ArrowLeft' && hasCompletion) {
                    clearSuggestion();
                    fetcher.pendingCompletion = null;
                }
                else if ((event.key === 'ArrowDown' || event.key === 'ArrowUp') && hasCompletion) {
                    clearSuggestion();
                }
                else if (event.key === 'Backspace') {
                    fetcher.pendingCompletion = null;
                    clearSuggestion();
                }
                else if (event.key === 'Enter' && !event.metaKey) {
                    event.preventDefault();
                    if (props.multiline) {
                        clearSuggestion();
                        editor.insertText('\n');
                    }
                }
                else if (event.key !== 'Shift') {
                    if (completion && !event.metaKey && completion.startsWith(event.key) && event.key.length === 1) {
                        updateCompletion(editor, completion.substring(1));
                        setCompletion(completion.substring(1));
                    }
                    else {
                        fetcher.pendingCompletion = null;
                        clearSuggestion();
                    }
                }
                if (props.onKeyDown) {
                    props.onKeyDown(event);
                }
            } })));
}
function isCompletionActive(editor) {
    var completion = editor.children.find(function (n) { return n.type === 'completion'; });
    return !!completion;
}
function isCursorAtEndOfDocument(editor) {
    var sel = editor.selection;
    if (!sel) {
        return false;
    }
    var node = sel.anchor.path[0];
    var offset = sel.anchor.offset;
    if (node !== editor.children.length - 1) {
        return false;
    }
    return offset === Node.string(editor.children[node]).length;
}
function acceptCompletion(editor, completion) {
    var isActive = isCompletionActive(editor);
    if (isActive) {
        removeCompletion(editor);
        editor.insertText(completion);
    }
}
function addCompletion(editor, completion) {
    var isActive = isCompletionActive(editor);
    var isCursorEnd = isCursorAtEndOfDocument(editor);
    if (!isActive && isCursorEnd) {
        Transforms.insertNodes(editor, {
            type: 'completion',
            children: [{ text: completion }],
        }, {
            at: [editor.children.length],
        });
    }
}
function removeCompletion(editor) {
    var isActive = isCompletionActive(editor);
    if (isActive) {
        Transforms.removeNodes(editor, {
            at: [editor.children.length - 1],
        });
    }
}
function updateCompletion(editor, newCompletion) {
    var isActive = isCompletionActive(editor);
    if (isActive && newCompletion.length > 0) {
        Transforms.delete(editor, {
            at: {
                anchor: { path: [editor.children.length - 1, 0], offset: 0 },
                focus: { path: [editor.children.length - 1, 0], offset: 1 },
            },
        });
    }
}
var serialize = function (nodes) {
    return nodes.map(function (n) { return Node.string(n); }).join('\n');
};
//# sourceMappingURL=SmartInput.js.map