var SmartInputFetcher = (function () {
    function SmartInputFetcher(addSuggestion, modelId, mockSuggestion) {
        var _this = this;
        this.addSuggestion = addSuggestion;
        this.modelId = modelId;
        this.mockSuggestion = mockSuggestion;
        this.sequenceId = 0;
        this.pendingCompletion = null;
        this.fetch = debounce(function (text, context) {
            var _a;
            if (text.length === 0) {
                return;
            }
            var seqId = ++_this.sequenceId;
            if ((_a = _this.pendingCompletion) === null || _a === void 0 ? void 0 : _a.suggestion) {
                var currExpectedText = _this.pendingCompletion.text + _this.pendingCompletion.suggestion;
                if (currExpectedText.startsWith(text)) {
                    return;
                }
            }
            _this.pendingCompletion = { text: text, sequenceId: seqId };
            if (_this.mockSuggestion) {
                var suggestion_1 = _this.mockSuggestion(text);
                if (suggestion_1) {
                    setTimeout(function () {
                        var _a;
                        if (((_a = _this.pendingCompletion) === null || _a === void 0 ? void 0 : _a.sequenceId) === seqId) {
                            _this.addSuggestion(text, suggestion_1);
                            _this.pendingCompletion.suggestion = suggestion_1;
                        }
                    }, getRandomInt(200, 1000));
                }
                return;
            }
            fetch('https://api.lydian.company/app/events', {
                method: 'POST',
                mode: 'cors',
                headers: {
                    'Content-Type': 'application/json',
                    Authorization: "Bearer ".concat(_this.modelId),
                },
                body: JSON.stringify({
                    model: _this.modelId,
                    session: '' + Date.now(),
                    input: text,
                    context: context,
                }),
            })
                .then(function (res) {
                if (res.status !== 200) {
                    console.error("Couldn't fetch suggestions");
                    return;
                }
                return res.json();
            })
                .then(function (data) {
                var _a;
                var suggestion = (data === null || data === void 0 ? void 0 : data.suggestion) || '';
                if (suggestion && ((_a = _this.pendingCompletion) === null || _a === void 0 ? void 0 : _a.sequenceId) === seqId) {
                    _this.addSuggestion(text, suggestion);
                    _this.pendingCompletion.suggestion = suggestion;
                }
            })
                .catch(function (e) { return console.error(e); });
        }, 200);
    }
    return SmartInputFetcher;
}());
export { SmartInputFetcher };
function getRandomInt(min, max) {
    min = Math.ceil(min);
    max = Math.floor(max);
    return Math.floor(Math.random() * (max - min + 1)) + min;
}
var debounce = function (callback, wait) {
    var timeoutId = null;
    return function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        window.clearTimeout(timeoutId);
        timeoutId = window.setTimeout(function () {
            callback.apply(null, args);
        }, wait);
    };
};
//# sourceMappingURL=SmartInputFetcher.js.map